package cn.iocoder.yudao.module.system.service.supplier;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.api.supplier.dto.SupplierRpcDTO;
import cn.iocoder.yudao.module.system.controller.admin.supplier.vo.SupplierPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.supplier.vo.SupplierSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.supplier.SupplierDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 供应商列表(OA已审核通过后插入) Service 接口
 *
 * @author 艾楷
 */
public interface SupplierService {

    /**
     * 创建供应商列表(OA已审核通过后插入)
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createSupplier(@Valid SupplierSaveReqVO createReqVO);

    /**
     * 更新供应商列表(OA已审核通过后插入)
     *
     * @param updateReqVO 更新信息
     */
    void updateSupplier(@Valid SupplierSaveReqVO updateReqVO);

    /**
     * 删除供应商列表(OA已审核通过后插入)
     *
     * @param id 编号
     */
    void deleteSupplier(Long id);

    /**
     * 获得供应商列表(OA已审核通过后插入)
     *
     * @param id 编号
     * @return 供应商列表(OA已审核通过后插入)
     */
    SupplierDO getSupplier(Long id);

    /**
     * 获得供应商列表(OA已审核通过后插入)分页
     *
     * @param pageReqVO 分页查询
     * @return 供应商列表(OA已审核通过后插入)分页
     */
    PageResult<SupplierDO> getSupplierPage(SupplierPageReqVO pageReqVO);

    /**
     * 新增编辑供应商信息
     *
     * @param supplier
     */
    void saveOrEdit(SupplierRpcDTO supplier);

    /**
     * 更新状态
     *
     * @param vo
     */
    void editStatus(SupplierSaveReqVO vo);

    /**
     * 获取所有供应商
     *
     * @return
     */
    List<SupplierDO> allList(Integer status);
}
