package cn.iocoder.yudao.module.system.service.worklog;


import cn.iocoder.yudao.module.system.dal.dataobject.worklog.LogReadDo;
import cn.iocoder.yudao.module.system.service.worklog.dto.LogReadUserRespDTO;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * 日志查看 Service 接口
 *
 * @author 符溶馨
 */
public interface LogReadService {

    /**
     * 创建日志规则
     *
     * @param createReqVO 创建信息
     */
    void createLogRule(@Valid List<LogReadUserRespDTO> createReqVO, Long logInstanceId, Long startUserId);

    /**
     * 获取日志可查看的用户编号组
     * @param logId 日志编号
     * @return 阅读状态，用户编号组
     */
    Map<Integer, List<Long>> getUserId(Long logId);

    /**
     * 更新日志阅读状态
     * @param logId 日志编号
     * @param userId 查看者用户编号
     */
    void updateReadStatus(Long logId, Long userId);

    /**
     * 获取当前登录用户可查看的日志列表
     * @param userId 用户编号
     * @return 日志列表
     */
    List<LogReadDo> getReadListByReadUserId(Long userId);

    /**
     * 获得指定日志的查看状态
     * @param logId 日志编号
     * @return 阅读状态
     */
    Boolean isReadByLogId(Long logId);

    /**
     * 获取当前登录用户可查看日志未读的数量
     * @param userId 用户编号
     * @return 未读数量
     */
    Long getUnRead(Long userId);

    /**
     * 一键已读
     * @param loginUserId 登录用户编号
     */
    void setRead(Long loginUserId);
}
